<div>
    <div class="table-responsive my-3">
        <table class="table table-bordered table-sm w-auto mx-auto">
            <thead>
                <tr>
                    {% for key in display_data.keys() %}
                        <th>{{ key.replace('_', ' ') | title }}</th>
                    {% endfor %}
                </tr>
            </thead>
            <tbody>
                <tr>
                    {% for value in display_data.values() %}
                        <td>{{ value }}</td>
                    {% endfor %}
                </tr>
            </tbody>
        </table>
    </div>

    {{ img_html | safe}}
</div>