<div class="dflex flex-column align-itmes-center">
    <table class="table table-striped table-bordered table-smd">
        <thead>
            <tr>
            {% for data_name in display_data.keys()%}
                <th scope="col">{{data_name.title()}}</th>
            {% endfor %}
            </tr>
        </thead>
        <tbody>
            <tr>
            {% for data_value in display_data.values()%}
                <td>{{ data_value }}</td>
            {% endfor %}
            </tr>
        </tbody>
    </table>
    {% if plot %}
    <div class="container text-center mt-4">
       {{ plot | safe }}
    </div>
    {% endif %}
</div>